/*  
	Extension:	Wikipedia formatting toolbar and context menu for Mozilla based browsers
	Author: 	Bananeweizen, http://www.bananeweizen.de
	Based on:	bbcode extension of Jed Brown, http://jedbrown.net/
				PastQuote extension by Ben Basson (Cusser), http://cusser.net/extensions/
*/

var prefInstance=null;
var wikipedia_stringBundle;
var wikipedia_Document = null;
var scrollTop;
var scrollHeight;
var currentBox;
var menuCreated=false;
var projectUrl="";
var projectLanguage="";

var wikipedia = {
	init: function() {
		wikipedia_setupDocument(document);
		//fix toolbar layout
		document.getElementById('wikipedia-toolbar').setAttribute("iconsize",document.getElementById('navigator-toolbox').getAttribute("iconsize"));
		document.getElementById('wikipedia-toolbar').setAttribute("mode",document.getElementById('navigator-toolbox').getAttribute("mode"));
		document.getElementById('wikipedia-specialpages-restricted').setAttribute("hidden",!getPreferences().getBoolPref("restrictedspecial"));
		wikipedia.setCurrentProject();
		wikipedia.generateMenus();	//TODO:should be created on need only!
		try{
			var menu = document.getElementById('contentAreaContextMenu');
			menu.addEventListener('popupshowing', wikipedia.showHide, false);
		}catch (e){
			//Foobar!
		}
		wikipedia.showHideTimer();
		return true;
	},
	
	showHide: function() {
		var kind=0;
		var activate=true;
		var condition="wiki";
		try {
			kind=getPreferences().getIntPref("activation.kind");
			condition=getPreferences().getCharPref("activation.condition");
			if (kind==1) {
				activate=(window.content.location.toString().indexOf(condition)>=0);
			}
		}
		catch (e) {
		}
		// load dynamic menu on first showing
		if (activate) {
			wikipedia.generateMenus();
		}
		
		//make toolbar and menu only visible if necessary
		document.getElementById('wikipedia-toolbar').hidden = !activate;
		document.getElementById('context-wikipedia').hidden = (document.getElementById('context-undo').hidden && document.getElementById('context-copy').hidden) || !activate;
		
		//set the label for the search selection menu item
		if (gContextMenu) {
			search=wikipedia.trim(gContextMenu.searchSelected());
			if (search.length>20) {
				search=search.substring(0,20)+"...";
			}
			var regEx = new RegExp ('%s', 'gi') ;
			searchLabel = 'Suche "%s"';
            searchLabel = searchLabel.replace(regEx, search);
			document.getElementById('wikipedia-search-selection').setAttribute("label",searchLabel);
		}
	},
	
	showHideTimer: function() {
		wikipedia.showHide();
		setTimeout("wikipedia.showHideTimer()",200);
	},
	
	getClipboard: function() {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip){
			return "";
		}
		
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans){
			return "";
		}

		trans.addDataFlavor("text/unicode");
		clip.getData(trans,clip.kGlobalClipboard);

		var str_clipboard=new Object();
		var strLength=new Object();
		try{
			trans.getTransferData("text/unicode",str_clipboard,strLength);
			if (str_clipboard){
				str_clipboard=str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
			}
		}catch (e){
			//alert("No text in the clipboard, please copy something first.");
		}
		return str_clipboard;
	},
	
	getSelection: function() {
		currentBox = document.commandDispatcher.focusedElement;
		scrollTop = currentBox.scrollTop;
		scrollHeight = currentBox.scrollHeight;

		//Get Selected text (if selected)
		var selStartPos = currentBox.selectionStart;
		var selEndPos = currentBox.selectionEnd;
		str = currentBox.value.substring(selStartPos, selEndPos);
		
		return str;
	},

	getSelectionAll: function() {
		currentBox.setSelectionRange(0,currentBox.value.length-1);
		return wikipedia.getSelection();
	},
	
	getForcedSelection: function(format) {
		var str = wikipedia.getSelection();
		if (str==null || str=="") {
			str=wikipedia.getInput("wikipedia.input."+format);
		}
		return str;
	},

	insert: function(someText) {
		wikipedia.getSelection();
		wikipedia.insertAtCursor(someText);
	},
	
	surround: function(format, left,right) {
		var str=wikipedia.getSelection();
		if (format!="") {
			str=wikipedia.getForcedSelection(format);
		}
		var leftTrim="";
		while ((str.length>0) && (str.substring(0,1) == ' ')) {
			leftTrim=leftTrim+" ";
			str=str.substring(1,str.length);
		}
		var rightTrim="";
		while ((str.length>0) && (str.substring(str.length-1,str.length) == ' ') ) {
			rightTrim=rightTrim+" ";
			str=str.substring(0,str.length-1);
		}
		wikipedia.insertAtCursor(leftTrim+left+str+right+rightTrim);
	},
	
	paragraph: function(format,markup) {
		var str=wikipedia.trim(wikipedia.getForcedSelection(format));
		wikipedia.insertAtCursor("\n"+markup+" "+str+" "+markup+"\n");
	},
	
	changeParagraphs: function(change) {
		var str=wikipedia.getSelection();
		if (str=="") {
			str=wikipedia.getSelectionAll();
		}
		var lines=str.split("\n");
		var insert="";
		for (i=0;i<lines.length;i=i+1) {
			var line=lines[i];
			if (line.indexOf("==")>=0) {
				if (change==1) {
					line = line.replace(new RegExp ('====', 'gi'), '=====') ;
					line = line.replace(new RegExp ('===', 'gi'), '====') ;
					line = line.replace(new RegExp ('==', 'gi'), '===') ;
				}
				else {
					line = line.replace(new RegExp ('===', 'gi'), '==') ;
					line = line.replace(new RegExp ('====', 'gi'), '===') ;
					line = line.replace(new RegExp ('=====', 'gi'), '====') ;
				}
			}
			if (i>0) {
				insert=insert+"\n";
			}
			insert=insert+line;
		}
		wikipedia.insertAtCursor(insert);
	},
	
	insertSignature: function() {
		var trailing="";
		wikipedia.getSelection();
		var selStartPos = currentBox.selectionStart;
		if (selStartPos>0) {
			var before = currentBox.value.substring(selStartPos-1, selStartPos);
			if ((before!=' ') && (before!='\n')) {
				trailing=" ";
			}
		}
		wikipedia.insert(trailing+'--~~~~');
	},
  
	insertAtCursor: function(aText) {
		try {
			var command = "cmd_insertText";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				params.setStringValue("state_data", aText);
				controller.doCommandWithParams(command, params);
				
				//restore position
				var nHeight = currentBox.scrollHeight - scrollHeight;
				currentBox.scrollTop = scrollTop + nHeight;
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
  
	insertList: function(listSign) {
		var str=wikipedia.getSelection();
		var lines=str.split("\n");
		var insert="";
		for (i=0;i<lines.length;i=i+1) {
			if (i>0) {
				insert=insert+"\n";
			}
			if (wikipedia.trim(lines[i])!="") {
				insert=insert+listSign+lines[i];
			}
		}
		if (str=="") {
			for (i=lines.length;i<3;i=i+1) {
				insert=insert+"\n"+listSign;
			}
		}
	  	wikipedia.insertAtCursor(insert);
	},
	
	insertDefinitionList: function() {
		var topic="...";
		var definition="...";
		var def=";"+topic+":"+definition+"\n";
		wikipedia.insertAtCursor(def+def);
	},
	
	removeList: function() {
		var lines=wikipedia.getSelection().split("\n");
		for (i=0;i<lines.length;i=i+1) {
			if (lines[i].length>0) {
				var c=lines[i].substring(0,1);
				if ((c=='*') || (c=='#') || (c==':') || (c==' ')) {
					lines[i]=lines[i].substr(1,lines[i].length-1);
				}
			}
		}
		var insert=lines[0];
		for (i=1;i<lines.length;i=i+1) {
			insert=insert+"\n"+lines[i];
		}
	  	wikipedia.insertAtCursor(insert);
	},
	
	insertTable: function() {
		var str=wikipedia.getSelection();
        var ok = [false];
        var params = [0,0,wikipedia.trim(str),false,false,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_table.xul", "WikiTable","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]){
        	var rows=Math.round(params[0]);
        	var columns=Math.round(params[1]);
        	var content=params[2];
        	var border=params[3];
        	var headline=params[4];
			var caption=wikipedia.trim(params[5]);
			var alignment=params[6];
			var color=params[7];
       	
        	if (headline) {
        		rows=1+rows;
        	}
        	var table="{|";
        	if (border) {
        		table=table+" border=1";
        	}
        	switch (alignment) {
        		case 1: 
        			table=table+" align=left";
        			break;
        		case 2: 
        			table=table+" align=center";
        			break;
        		case 3: 
        			table=table+" align=right";
        			break;
        	}
        	table=table+"\n";
        	if (caption!="") {
        		table=table+"|+"+caption+"\n";
        	}
        	for (r=0;r<rows;r=r+1) {
        		var cellStart="|";
        		if (headline && (r==0)) {
        			cellStart="!";
        		}
        		for (c=0;c<columns;c=c+1) {
        			table=table+cellStart+content+"\n";
        		}
        		if (r<rows-1) {
	        		table=table+"|----";
					if (color!="") {
		        		table=table+" bgcolor="+color;
        			}
	        		table=table+"\n";
	        	}
        	}
        	table=table+"|}\n";
			wikipedia.insertAtCursor(table);
        }
	},
	
	insertImage: function() {
		var str=wikipedia.trim(wikipedia.getSelection());
        var ok = [false];
        // use selected string as either image name or description
        var imageName="";
        var imageDesc="";
        if (str.indexOf(".")>=0) {
        	imageName=str;
        }
        else {
        	imageDesc=str;
        }
        // call dialog
        var params = [imageName,imageDesc,true,false,false,false,0,false,false,false,0,0,0,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_image.xul", "WikiImage","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]){
        	var file=wikipedia.trim(params[0]);
        	var description=wikipedia.trim(params[1]);
        	var thumb=params[2];
        	var thumbsized=params[3];
        	var framed=params[4];
        	var full=params[5];
        	var pixels=params[6];
        	var alignment=params[7];
        	var imageNamespace=wikipedia.getLanguageString("image");
        	if (imageNamespace=="") {
        		imageNamespace="Image";
        	}
        	var image="[["+imageNamespace+":"+file;
        	if (framed) {
        		image=image+"|framed";
        	}
        	else if (thumb) {
        		image=image+"|thumb";
        	}
        	else if (thumbsized) {
        		image=image+"|thumb|"+pixels+"px";
        	}
        	switch (alignment) {
	        	case 1: 
	        		image=image+"|left";
	        		break;
	        	case 2:
	        		image=image+"|none";
	        		break;
	        	case 3:
	        		image=image+"|right";
	        		break;
        	}
        	image=image+"|"+description;
        	image=image+"]]";
        	if (alignment==2) {
        		image="<center>"+image+"</center>";
        	}
			wikipedia.insertAtCursor(image);
        }
	},
	
	insertLink: function(borderLeft, borderRight, separator) {
		var str=wikipedia.getSelection();
        var ok = [false];
        var left=blankLeft(str);
        var right=blankRight(str);
		str=wikipedia.trim(str);
		var str_target="";
		var str_visible="";
		var showDialog=getPreferences().getBoolPref("linkdialog")
		if ((!showDialog) || (getPreferences().getIntPref("selectionlink")==0)) {
			str_target=str;
		}
		else {
			str_visible=str;
		}
        var params = [str_target,str_visible,"",true,0,0,0,0];
        if (showDialog) {
	        window.openDialog("chrome://wikipedia/content/wikipedia_link.xul", "WikiLink","chrome,modal,centerscreen,resizable",ok,params);
	    }
        if ((ok[0]) || (!showDialog)){
        	var target=wikipedia.trim(params[0]);
        	var label=wikipedia.trim(params[1]);
        	var interwiki=wikipedia.trim(params[2]);
        	var link='[[';
        	if (interwiki!="") {
        		link=link+":"+interwiki+":";
        	}
        	link=link+target;
        	if (label!="") {
        		link=link+'|'+label;
        	}
        	link=link+']]';
			wikipedia.insertAtCursor(left+link+right);
        }
	},
	
	insertWebLink: function() {
		var str=wikipedia.getSelection();
        var ok = [false];
        var left=blankLeft(str);
        var right=blankRight(str);
		str=wikipedia.trim(str);
		var str_target="http://";
		var str_visible="";
		if ((str.indexOf("http")>=0) || (str.indexOf("www.")>=0)) {
			str_target=str;
		}
		else {
			str_visible=str;
		}
        var params = [str_target,str_visible,"",false,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/wikipedia_link.xul", "WikiLink","chrome,modal,centerscreen,resizable",ok,params);
        if (ok[0]){
        	var target=wikipedia.trim(params[0]);
        	if (target.indexOf("http")!=0) {
        		target="http://"+target;
        	}
        	var label=wikipedia.trim(params[1]);
        	var link="["+target;
        	if (label!="") {
        		link=link+" "+label;
        	}
        	link=link+"]";
			wikipedia.insertAtCursor(left+link+right);
		}
	},
	
	trim: function(input) {
		var s=input;
		if (s==null || s=="") {
			return "";
		}
		while (s.substring(0,1) == ' ') {
			s = s.substring(1,s.length);
		}
		while (s.substring(s.length-1,s.length) == ' ') {
			s = s.substring(0,s.length-1);
		}
		return s;
	},

	getUsername: function() {
		var username="";
		try {
			username=getPreferences().getCharPref("username");
		}
		catch(e) {
		}
		if (username=="") {
			alert(wikipedia.getString("wikipedia.settings.username.undefined"));
			wikipedia.toolbarOptions();
		}
		return getPreferences().getCharPref("username");		
	},
	
	getUrl: function(language) {
		var nodelist = wikipedia.getNodes("chrome://wikipedia/content/wiki_projects.xml","/languages/language[code='"+language+"']");
		node=nodelist[0];
		var url = document.evaluate("wikipedia/text()",node,null,0,null);
		try {
			url = url.iterateNext().nodeValue;
		}
		catch(e) {
			url="http://"+language+'.wikipedia.org/wiki/';
		}
		return url;
	},
	
	getCurrentUrl: function() {
		return projectUrl;
	},
	
	getCurrentLanguage: function() {
		return projectLanguage;
	},
	
	setLocation: function(name) {
		var wp_homepage = wikipedia.getCurrentUrl();
		var url="";
		switch (name) {
			case 'wikipedia':
				url=wp_homepage;
				break;
			case 'toolbar':
				url='http://www.bananeweizen.de/mozilla/wikipedia/firefox_wikipedia.html';
				break;
			case 'user':
				var username=wikipedia.getUsername();
				if (username!="") {
					url=wp_homepage+'User:'+username;
				}
				break;
			case 'usertalk':
				var username=wikipedia.getUsername();
				if (username!="") {
					url=wp_homepage+'User_talk:'+username;
				}
				break;
			case 'contributions':
				var username=wikipedia.getUsername();
				if (username!="") {
					url=wp_homepage+'Special:Contributions?target='+username;
				}
				break;
		}
		if (url!="") {
			window._content.document.location=url;
		}
	},
	
	specialPage:function(pageName) {
		var wp_homepage = wikipedia.getCurrentUrl();
		var url=wp_homepage+'Special:'+pageName;
		window._content.document.location=url;
	},
	
	search:function(value) {
		value=wikipedia.trim(value);
		if (value!="") {
			window._content.document.location=wikipedia.getCurrentUrl()+"Special:Search?search="+value;
		}
	},
	
	searchToolbar: function() {
		wikipedia.search(document.getElementById('wikipedia-search').value);
	},
	
	searchSelection: function() {
		if (gContextMenu) {
			wikipedia.search(gContextMenu.searchSelected());
		}
	},

	toolbarOptions: function() {
        var ok = [false];
        var params = [];
        window.openDialog("chrome://wikipedia/content/wikipedia_preferences.xul", "WikiOptions","chrome,modal,centerscreen,resizable",ok,params);
	},
	
	getString:function(name) {
		if (wikipedia_stringBundle==null) {
			wikipedia_stringBundle = document.getElementById("wikipediastrings");
		}
		return wikipedia_stringBundle.getString(name);
	},
	
	getInput: function(textId) {
		var dialogText=wikipedia.getString(textId);
		var newtext = prompt(dialogText);
		return newtext;
	},
	
	getLanguages: function() {
		try {
			var langlist = new Array();
			var nodelist = wikipedia.getNodes("chrome://wikipedia/content/wiki_projects.xml","/languages/language");
			for(var i=0; i < nodelist.length; i++) {
				node=nodelist[i];
				var code = document.evaluate("code/text()",node,null,0,null);
				code = code.iterateNext().nodeValue;
				var name = document.evaluate("name/text()",node,null,0,null);
				name = name.iterateNext().nodeValue;
				var english = document.evaluate("name_english/text()",node,null,0,null);
				english = english.iterateNext().nodeValue;

				var entry=new Array();
				entry.push(code);
				entry.push(name);
				entry.push(english);
				langlist.push(entry);
			}
			return langlist;
		} catch (e) {
			dump(e);
		}
	},

	getLanguageString: function(tagName) {
		try {
			var code="en";
			try {
				code=getPreferences().getCharPref("language");
			}
			catch (e) {
			}
			var nodelist = wikipedia.getNodes("chrome://wikipedia/content/wiki_projects.xml","/languages/language[code='"+code+"']");
			if (nodelist.length==0) {
				return "";
			}
			node=nodelist[0];
			var langString = document.evaluate(tagName+"/text()",node,null,0,null);
			langString = langString.iterateNext().nodeValue;
			return langString;
		} catch (e) {
			dump(e);
			return "";
		}
	},
	
	getNodes: function(url, nodeSelector) {
		var content = getContentsFromURL(url);
		var uConv = Components.classes['@mozilla.org/intl/scriptableunicodeconverter'].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
		uConv.charset = "UTF-8";
		var content = uConv.ConvertToUnicode(content);
		
		var parser = new DOMParser();
		var domtree = parser.parseFromString(content, "text/xml");
		var nodes = document.evaluate(nodeSelector,domtree,null,0,null);
		var nodelist = new Array();
		var xpathnode = nodes.iterateNext();
		while (xpathnode) {
			nodelist.push(xpathnode);
			xpathnode = nodes.iterateNext();
		}
		return nodelist;
	},
	
	generateMenuTemplates: function(number) {
		try {
			var popup = document.getElementById('popupTemplates'+number);
/*			
			alert(popup);
			templates=popup.childNodes;
			alert(templates.length);
			for (i=templates.length-1;i>3;i--) {
				template=templates[i];
				alert(template);
				popup.removeChild(template);
			}
*/			
			var nodelist = wikipedia.getNodes("chrome://wikipedia/locale/templates.xml","/templates/template");
			for(var i=0; i < nodelist.length; i++) {
				node=nodelist[i];
				var label = document.evaluate("label/text()",node,null,0,null);
				label = label.iterateNext().nodeValue;
				if (label=="-") {
					var newElement = document.createElement('menuseparator');
					popup.appendChild(newElement);
				}
				else {
					var insert = document.evaluate("insert/text()",node,null,0,null);
					insert = insert.iterateNext().nodeValue;
					var tooltip = document.evaluate("tooltip/text()",node,null,0,null);
					if (tooltip!=null) {
						tooltip = tooltip.iterateNext().nodeValue;
					}
			
					var newElement = document.createElement('menuitem');
					newElement.setAttribute('id', 'template'+i);
					newElement.setAttribute('label', label);
					newElement.setAttribute('oncommand', 'wikipedia.insert(\''+insert+'\')');
					if (tooltip!=null) {
						newElement.setAttribute('tooltiptext', tooltip);
					}
					popup.appendChild(newElement);
				}
			}
		} catch (e) {
			dump(e);
		}
	},
	
	generateMenuQuotationMarks: function(number) {
		try {
			var popup = document.getElementById('popupQuotations'+number);
			var nodelist = wikipedia.getNodes("chrome://wikipedia/content/quotation_marks.xml","/quotation_marks/quotation[code='"+wikipedia.getCurrentLanguage()+"']");
			for(var i=0; i < nodelist.length; i++) {
				node=nodelist[i];
				wikipedia.addQuote("standard_double", node, popup);
				wikipedia.addQuote("standard_single", node, popup);
				wikipedia.addQuote("alternative_double", node, popup);
				wikipedia.addQuote("alternative_single", node, popup);
				popup.appendChild(document.createElement('menuseparator'));
			}

			var nodelist = wikipedia.getNodes("chrome://wikipedia/content/quotation_marks.xml","/quotation_marks/quotation");
			for(var i=0; i < nodelist.length; i++) {
				node=nodelist[i];
				var language = document.evaluate("language/text()",node,null,0,null);
				language = language.iterateNext().nodeValue;
				var quoteLangMenu = document.createElement('menu');
				quoteLangMenu.setAttribute('label', language);
				popup.appendChild(quoteLangMenu);
				var quoteLangPopup = document.createElement('menupopup');
				quoteLangMenu.appendChild(quoteLangPopup);
				
				wikipedia.addQuote("standard_double", node, quoteLangPopup);
				wikipedia.addQuote("standard_single", node, quoteLangPopup);
				wikipedia.addQuote("alternative_double", node, quoteLangPopup);
				wikipedia.addQuote("alternative_single", node, quoteLangPopup);
			}

		} catch (e) {
			dump(e);
		}
	},

	generateMenus: function(){
		if (false==menuCreated) {
			for (number=1;number<=2;number++) {
				wikipedia.generateMenuTemplates(number);
				wikipedia.generateMenuQuotationMarks(number);
			}
			menuCreated=true;
		}
		return true;
	},

	addQuote: function(quoteName, node, quoteLangPopup) {
		var quote = document.evaluate(quoteName+"/text()",node,null,0,null);
		quote = quote.iterateNext();
		if (quote!=null) {
			quote=quote.nodeValue;
			var left=quote.substring(0,(quote.length-1)/2);
			var right=quote.substring((quote.length+1)/2,quote.length);
			var quoteItem = document.createElement('menuitem');
			quoteItem.setAttribute('label', quote+" - "+wikipedia.getString("wikipedia.quotation."+quoteName));
			quoteItem.setAttribute('oncommand', 'wikipedia.surround(\'quotation\',\''+left+'\',\''+right+'\')');
			quoteLangPopup.appendChild(quoteItem);
		}
	},
	
	showProjectMenu: function(menuid) {
		var children = menuid.childNodes;
		for (var j = children.length-1; j >= 0; j--) {
			menuid.removeChild(children[j]);
		}
		//add standard wikis
		count=getPreferences().getIntPref("active.count");
		for (i=0;i<count;i++) {
			url=getPreferences().getCharPref("active.url"+i);
			label=getPreferences().getCharPref("active.label"+i);
			language=getPreferences().getCharPref("active.language"+i);
			menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", label);
			menuitem.setAttribute("oncommand", "wikipedia.selectProject(\""+label+"\",\""+url+"\",\""+language+"\");");
			menuid.appendChild(menuitem);
		}
		//add user wikis
		count=getPreferences().getIntPref("userwiki.count");
		for (i=0;i<count;i++) {
			label=getPreferences().getCharPref("userwiki."+i+".label");
			url=getPreferences().getCharPref("userwiki."+i+".url");
			language="";
			menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", label);
			menuitem.setAttribute("oncommand", "wikipedia.selectProject(\""+label+"\",\""+url+"\",\""+language+"\");");
			menuid.appendChild(menuitem);
		}
		return;
	},
	
	selectProject: function(label,url,language) {
		var currentUrl=window._content.document.location.toString();
		if ((currentUrl=="") || (currentUrl.indexOf(url)!=0)) {
			document.getElementById("project-name").setAttribute("value",label);
			if (getPreferences().getBoolPref("homeonchange") && (projectUrl!='')) {
				projectUrl=url;
				projectLanguage=language;
				wikipedia.setLocation('wikipedia');
			}
			projectUrl=url;
			projectLanguage=language;
			getPreferences().setCharPref("active.label",label);
			getPreferences().setCharPref("active.url",url);
			getPreferences().setCharPref("active.language",language);
			// rebuild menus
//			menuCreated=false;
//			generateMenus();
		}
	},
	
	setCurrentProject: function() {
		label=getPreferences().getCharPref("active.label");
		url=getPreferences().getCharPref("active.url");
		language=getPreferences().getCharPref("active.language");
		wikipedia.selectProject(label,url,language);
	}
}

function blankLeft(s) {
	if (s==null || s=="") {
		return "";
	}
	var r="";
	while (s.substring(0,1) == ' ') {
		r=r+s.substring(0,1);
		s = s.substring(1,s.length);
	}
	return r;
}

function blankRight(s) {
	if (s==null || s=="") {
		return "";
	}
	var r="";
	while (s.substring(s.length-1,s.length) == ' ') {
		r = s.substring(s.length-1,s.length);
		s = s.substring(0,s.length-1);
	}
	return r;
}

function getPreferences() {
	if (!prefInstance) {
		try {
			var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			prefInstance = prefService.getBranch("wikipedia."); // preferences wikipedia node
			prefInstance.setIntPref("lastversion",60);
		} catch (e) {
			alert(e);
		}
	}
	return prefInstance;
}
//May not be needed, but just incase.
function wikipedia_setupDocument(aDoc)
{
	wikipedia_Document = aDoc;
	wikipedia_stringBundle = aDoc.getElementById("wikipediastrings");
}

function readFile(str_Filename)
{
	try{
		var obj_File = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		obj_File.initWithPath(str_Filename);
		
		var obj_InputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
		obj_InputStream.init(obj_File,0x01,0444,null);
		
		var obj_ScriptableIO = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
		obj_ScriptableIO.init(obj_InputStream);
	} catch (e) {
		alert(e);
	}
	
	try {
		var str = obj_ScriptableIO.read(obj_File.fileSize-1);
	} catch (e) {
		dump(e);
	}
	
	obj_ScriptableIO.close();
	obj_InputStream.close();
	return str;
}

function getContentsFromURL(aURL){
	var ioService=Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var scriptableStream=Components.classes["@mozilla.org/scriptableinputstream;1"].getService(Components.interfaces.nsIScriptableInputStream);
	
	var channel=ioService.newChannel(aURL,null,null);
	var input=channel.open();
	scriptableStream.init(input);
	var str=scriptableStream.read(input.available());
	scriptableStream.close();
	input.close();
	return str;
}

//Make sure wikipedia loads on start-up of browser.
window.addEventListener('load', wikipedia.init, false);